import QtQuick 2.5
import Fly.Components 1.0
import Fly.Gallery 1.0

Item {
    id: bigImageContainer
    property url mediaPath
    property real minScale: 1
    property real maxScale: 10
    property alias autoTransform: bigImage.autoTransform
    property bool exitOnDoubleClick: false
    property bool swipe: false
    signal clicked
    signal nextRequest
    signal previousRequest

    state: "view-image"
    states : [
        State {
            name: "view-image";
            PropertyChanges { target: bigImage; isZooming: false; zoom: 1.0; lastTouchPoint: Qt.point(0, 0) }
        },
        State {
            name: "scale-image";
            PropertyChanges { target: bigImage; isZooming: true }
        }
    ]

    Image {
        id: bigImage

        property point lastTouchPoint: Qt.point(0, 0)

        fillMode: Image.PreserveAspectFit
        width: parent.width * zoom
        height: parent.height * zoom
        asynchronous: true
        cache: false
        sourceSize: Qt.size(4096, 0)
        source: loader.outputUrl

        property bool isZooming: false
        property real zoom: 1.0

        x: bigImage.lastTouchPoint.x
        y: bigImage.lastTouchPoint.y

        Behavior on zoom { NumberAnimation { duration: 200 } }
        Behavior on x { NumberAnimation { duration: 200 } }
        Behavior on y { NumberAnimation { duration: 200 } }

        PinchArea {
            id: imagePinchArea

            property real initialScale: 1.0

            pinch.dragAxis: Pinch.XAndYAxis
            anchors.fill: parent

            onPinchStarted: {
                bigImageContainer.state = "scale-image"
                initialScale = bigImage.zoom;
            }

            onPinchUpdated: {
                var nextZoom = Math.max(minScale, Math.min(maxScale, initialScale * pinch.scale));
                imageMouseArea.setZoom(pinch.center.x, pinch.center.y, nextZoom);
            }

            MouseArea {
                id: imageMouseArea
                anchors.fill: parent
                hoverEnabled: true
                scrollGestureEnabled: false  // 2-finger-flick gesture should pass through to the ListView
                property point pressedPos: Qt.point(0,0)
                property bool ignoreClick: false

                drag {
                    target: bigImage
                    axis: Drag.XAndYAxis
                    minimumX: bigImageContainer.width - bigImage.width; maximumX: 0
                    minimumY: bigImageContainer.height - bigImage.height; maximumY: 0
                }

                onClicked: {
                    if(!ignoreClick)
                        bigImageContainer.clicked();
                }

                onWheel: {
                    bigImageContainer.state = "scale-image"
                    var delta = bigImage.zoom * wheel.angleDelta.y / 120.0 / 2.0;
                    var nextZoom = Math.max(minScale, Math.min(maxScale, bigImage.zoom + delta));
                    setZoom(wheel.x, wheel.y, nextZoom);
                }

                onDoubleClicked: {
                    if(bigImageContainer.exitOnDoubleClick) {
                        Qt.quit();

                    } else {
                        mouse.accepted = false;
                        var nextZoom = 1;
                        if(bigImage.zoom > minScale) {
                            bigImageContainer.state = "view-image";
                            nextZoom = minScale;
                        } else {
                            nextZoom = Math.min(4, maxScale);
                            bigImageContainer.state = "scale-image";
                        }
                        setZoom(mouse.x, mouse.y, nextZoom);
                    }
                }

                onPressed: {
                    ignoreClick = false;
                    pressedPos = Qt.point(mouse.x, mouse.y);
                }

                onReleased: {
                    var deltaX = mouse.x - pressedPos.x;
                    var deltaY = mouse.y - pressedPos.y;
                    var thresholdX = bigImageContainer.width * 0.3;
                    var thresholdY = bigImageContainer.width * 0.2;
                    if(Math.abs(deltaY) < thresholdY && bigImageContainer.swipe) {
                        if(atTheLeftAdge() && deltaX > thresholdX) {
                            ignoreClick = true;
                            bigImageContainer.previousRequest();

                        } else if(atTheRightAdge() && deltaX < -thresholdX) {
                            ignoreClick = true;
                            bigImageContainer.nextRequest();
                        }
                    }
                }

                function setZoom(x, y, nextZoom) {
                    if(nextZoom !== bigImage.zoom) {
                        var nextX = x + bigImage.x - x * nextZoom / bigImage.zoom;
                        var nextY = y  + bigImage.y - y * nextZoom / bigImage.zoom;

                        nextX = Math.max(nextX, bigImageContainer.width - bigImageContainer.width * nextZoom);
                        nextY = Math.max(nextY, bigImageContainer.height - bigImageContainer.height * nextZoom);

                        bigImage.lastTouchPoint = Qt.point(Math.min(nextX, 0), Math.min(nextY, 0));
                        bigImage.zoom = nextZoom;
                    }
                }

                function atTheLeftAdge() {
                    return bigImageContainer.x === bigImage.x;
                }

                function atTheRightAdge() {
                    return bigImageContainer.x + bigImageContainer.width === bigImage.x + bigImage.width;
                }
            }
        }
    }

    FlyColoredImage {
        id: stateIcon
        visible: loader.running
        imagePath: "qrc:/icons/state_loading.svg"
        anchors { centerIn: parent }
        height: 150 * dpiRatio
        width: height
        antialiasing: true

        RotationAnimation on rotation {
            loops: Animation.Infinite
            from: 0
            to: -360
            running: loader.running
            duration: 1500
        }
    }

    ImageLoader {
        id: loader
    }

    onMediaPathChanged: {
        if(mediaPath) {
            bigImage.source.length = 0;
            loader.start(mediaPath, bigImage.sourceSize);
        }
    }
}
