import QtQuick 2.5
import Fly.Components 1.0
import Fly.Gallery 1.0

Rectangle {
    id: pdfContainer
    property url mediaPath
    property real minScale: 1
    property real maxScale: 10
    property var model
    property int page
    property alias showPageNumber: pageNumber.visible
    property bool exitOnDoubleClick: false
    property bool swipe: false
    signal clicked
    signal nextRequest
    signal previousRequest

    state: "view-pdf"
    states : [
        State {
            name: "view-pdf";
            PropertyChanges { target: pdfPage; isZooming: false; zoom: 1.0; lastTouchPoint: Qt.point(0, 0) }
        },
        State {
            name: "scale-pdf";
            PropertyChanges { target: pdfPage; isZooming: true }
        }
    ]

    color: "white"

    Image {
        id: pdfPage

        property point lastTouchPoint: Qt.point(0, 0)
        fillMode: Image.PreserveAspectFit
        width: parent.width * zoom
        height: parent.height * zoom
        asynchronous: true
        cache: false
        sourceSize: Qt.size(4096, 0)
        source: imageUrl
        antialiasing: false

        property bool isZooming: false
        property real zoom: 1.0

        x: pdfPage.lastTouchPoint.x
        y: pdfPage.lastTouchPoint.y

        Behavior on zoom { NumberAnimation { duration: 200 } }
        Behavior on x { NumberAnimation { duration: 200 } }
        Behavior on y { NumberAnimation { duration: 200 } }

        PinchArea {
            id: imagePinchArea

            property real initialScale: 1.0

            pinch.dragAxis: Pinch.XAndYAxis
            anchors.fill: parent

            onPinchStarted: {
                pdfContainer.state = "scale-pdf"
                initialScale = pdfPage.zoom;
            }

            onPinchUpdated: {
                var nextZoom = Math.max(minScale, Math.min(maxScale, initialScale * pinch.scale));
                imageMouseArea.setZoom(pinch.center.x, pinch.center.y, nextZoom);
            }

            MouseArea {
                id: imageMouseArea
                anchors.fill: parent
                hoverEnabled: true
                scrollGestureEnabled: false  // 2-finger-flick gesture should pass through to the ListView
                property point pressedPos: Qt.point(0,0)
                property bool ignoreClick: false

                drag {
                    target: pdfPage
                    axis: Drag.XAndYAxis
                    minimumX: pdfContainer.width - pdfPage.width; maximumX: 0
                    minimumY: pdfContainer.height - pdfPage.height; maximumY: 0
                }

                onClicked: {
                    if(!ignoreClick)
                        pdfContainer.clicked();
                }

                onWheel: {
                    pdfContainer.state = "scale-pdf"
                    var delta = pdfPage.zoom * wheel.angleDelta.y / 120.0 / 2.0;
                    var nextZoom = Math.max(minScale, Math.min(maxScale, pdfPage.zoom + delta));
                    setZoom(wheel.x, wheel.y, nextZoom);
                }

                onPressed: {
                    ignoreClick = false;
                    pressedPos = Qt.point(mouse.x, mouse.y);
                }

                onReleased: {
                    var deltaX = mouse.x - pressedPos.x;
                    var deltaY = mouse.y - pressedPos.y;
                    var thresholdX = pdfContainer.width * 0.3;
                    var thresholdY = pdfContainer.width * 0.2;
                    if(Math.abs(deltaY) < thresholdY && pdfContainer.swipe) {
                        if(atTheLeftAdge() && deltaX > thresholdX) {
                            ignoreClick = true;
                            pdfContainer.previousRequest();

                        } else if(atTheRightAdge() && deltaX < -thresholdX) {
                            ignoreClick = true;
                            pdfContainer.nextRequest();
                        }
                    }
                }

                onDoubleClicked: {
                    if(pdfContainer.exitOnDoubleClick) {
                        Qt.quit();

                    } else {
                        mouse.accepted = false;
                        var nextZoom = 1;
                        if(pdfPage.zoom > minScale) {
                            pdfContainer.state = "view-pdf";
                            nextZoom = minScale;
                        } else {
                            nextZoom = Math.min(4, maxScale);
                            pdfContainer.state = "scale-pdf";
                        }
                        setZoom(mouse.x, mouse.y, nextZoom);
                    }
                }

                function setZoom(x, y, nextZoom) {
                    if(nextZoom !== pdfPage.zoom) {
                        var nextX = x + pdfPage.x - x * nextZoom / pdfPage.zoom;
                        var nextY = y  + pdfPage.y - y * nextZoom / pdfPage.zoom;

                        nextX = Math.max(nextX, pdfContainer.width - pdfContainer.width * nextZoom);
                        nextY = Math.max(nextY, pdfContainer.height - pdfContainer.height * nextZoom);

                        pdfPage.lastTouchPoint = Qt.point(Math.min(nextX, 0), Math.min(nextY, 0));
                        pdfPage.zoom = nextZoom;
                    }
                }

                function atTheLeftAdge() {
                    return pdfContainer.x === pdfPage.x;
                }

                function atTheRightAdge() {
                    return pdfContainer.x + pdfContainer.width === pdfPage.x + pdfPage.width;
                }
            }
        }

        Rectangle {
            id: pageNumber
            color: "black"
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 10 * dpiRatio
            width: 50 * dpiRatio
            height: 25 * dpiRatio
            opacity: 0.5
            visible: false

            FlyLabel {
                anchors.fill: parent
                anchors.margins: 2
                text: pdfContainer.page + 1
                color: "white"
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter
            }
        }

        Component.onCompleted: {
            model.renderPage(page, Qt.size(50, 0));
        }

        onWidthChanged: renderTimer.restart();
        onHeightChanged: renderTimer.restart();


        Timer {
            id: renderTimer
            interval: 300
            repeat: false
            onTriggered: {
                model.renderPage(page, Qt.size(pdfPage.width, pdfPage.height));
            }
        }

    }
}
